@@ -1,10 +1,13 @@
 use swc_atoms::js_word;
-use swc_common::{Spanned, DUMMY_SP};
+use swc_common::{util::take::Take, Spanned, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::perf::Check;
 use swc_ecma_transforms_macros::fast_path;
-use swc_ecma_utils::{contains_this_expr, private_ident, quote_ident, ExprFactory};
-use swc_ecma_visit::{noop_fold_type, noop_visit_type, Fold, FoldWith, Node, Visit};
+use swc_ecma_utils::{contains_this_expr, prepend, private_ident};
+use swc_ecma_visit::{
+    noop_fold_type, noop_visit_mut_type, noop_visit_type, Fold, FoldWith, InjectVars, Node, Visit,
+    VisitMut, VisitMutWith,
+};
 
 /// Compile ES2015 arrow functions to ES5
 ///
@@ -54,19 +57,28 @@ use swc_ecma_visit::{noop_fold_type, noop_visit_type, Fold, FoldWith, Node, Visi
 /// };
 /// console.log(bob.printFriends());
 /// ```
-pub fn arrow() -> impl Fold {
+pub fn arrow() -> impl Fold + InjectVars {
     Arrow::default()
 }
 
 #[derive(Default)]
 struct Arrow {
     in_arrow: bool,
+    vars: Vec<VarDeclarator>,
 }
 
 #[fast_path(ArrowVisitor)]
 impl Fold for Arrow {
     noop_fold_type!();
 
+    fn fold_constructor(&mut self, c: Constructor) -> Constructor {
+        let in_arrow = self.in_arrow;
+        self.in_arrow = false;
+        let res = c.fold_children_with(self);
+        self.in_arrow = in_arrow;
+        res
+    }
+
     fn fold_expr(&mut self, e: Expr) -> Expr {
         match e {
             Expr::Arrow(ArrowExpr {
@@ -78,9 +90,7 @@ impl Fold for Arrow {
                 type_params,
                 return_type,
             }) => {
-                let used_this = contains_this_expr(&body);
-
-                let mut params: Vec<Param> = params
+                let params: Vec<Param> = params
                     .fold_with(self)
                     .into_iter()
                     .map(|pat| Param {
@@ -90,10 +100,14 @@ impl Fold for Arrow {
                     })
                     .collect();
 
-                // If we aren't already in an arrow expression, check if there is
-                // any arguments references within this function. If so, we need to
-                // replace them so that they refer to the parent function environment.
-                let (body, used_arguments) = if !self.in_arrow {
+                let mut arguments_var = None;
+
+                let in_arrow = self.in_arrow;
+                self.in_arrow = true;
+                let body = body.fold_with(self);
+                self.in_arrow = in_arrow;
+
+                let mut body = {
                     let mut arguments_replacer = ArgumentsReplacer {
                         arguments: private_ident!("_arguments"),
                         found: false,
@@ -101,26 +115,41 @@ impl Fold for Arrow {
                     let body = body.fold_with(&mut arguments_replacer);
 
                     if arguments_replacer.found {
-                        params.insert(
-                            0,
-                            Param {
-                                span: DUMMY_SP,
-                                decorators: Default::default(),
-                                pat: Pat::Ident(BindingIdent::from(
-                                    arguments_replacer.arguments.clone(),
-                                )),
-                            },
-                        );
+                        arguments_var = Some(arguments_replacer.arguments.clone());
                     }
-                    (body, arguments_replacer.found)
-                } else {
-                    (body, false)
+                    body
                 };
 
-                let in_arrow = self.in_arrow;
-                self.in_arrow = true;
-                let body = body.fold_with(self);
-                self.in_arrow = in_arrow;
+                let used_this = contains_this_expr(&body);
+
+                let this_id = if used_this {
+                    Some(private_ident!("_this"))
+                } else {
+                    None
+                };
+
+                if let Some(this_id) = this_id.clone() {
+                    self.vars.push(VarDeclarator {
+                        span: DUMMY_SP,
+                        name: Pat::Ident(this_id.clone().into()),
+                        init: Some(Box::new(Expr::This(ThisExpr { span: DUMMY_SP }))),
+                        definite: false,
+                    });
+
+                    body.visit_mut_with(&mut ThisReplacer { to: &this_id })
+                }
+
+                if let Some(id) = arguments_var.clone() {
+                    self.vars.push(VarDeclarator {
+                        span: DUMMY_SP,
+                        name: Pat::Ident(id.into()),
+                        init: Some(Box::new(Expr::Ident(Ident::new(
+                            js_word!("arguments"),
+                            DUMMY_SP,
+                        )))),
+                        definite: false,
+                    });
+                }
 
                 let fn_expr = Expr::Fn(FnExpr {
                     ident: None,
@@ -145,34 +174,12 @@ impl Fold for Arrow {
                     },
                 });
 
-                if !used_this && !used_arguments {
-                    return fn_expr;
-                }
-
-                let mut args = vec![ThisExpr { span: DUMMY_SP }.as_arg()];
-                if used_arguments {
-                    args.push(quote_ident!("arguments").as_arg());
-                }
-
-                Expr::Call(CallExpr {
-                    span,
-                    callee: fn_expr.make_member(quote_ident!("bind")).as_callee(),
-                    args,
-                    type_args: Default::default(),
-                })
+                return fn_expr;
             }
             _ => e.fold_children_with(self),
         }
     }
 
-    fn fold_constructor(&mut self, c: Constructor) -> Constructor {
-        let in_arrow = self.in_arrow;
-        self.in_arrow = false;
-        let res = c.fold_children_with(self);
-        self.in_arrow = in_arrow;
-        res
-    }
-
     fn fold_function(&mut self, f: Function) -> Function {
         let in_arrow = self.in_arrow;
         self.in_arrow = false;
@@ -180,6 +187,50 @@ impl Fold for Arrow {
         self.in_arrow = in_arrow;
         res
     }
+
+    fn fold_module_items(&mut self, stmts: Vec<ModuleItem>) -> Vec<ModuleItem> {
+        let mut stmts = stmts.fold_children_with(self);
+        if !self.vars.is_empty() {
+            prepend(
+                &mut stmts,
+                ModuleItem::Stmt(Stmt::Decl(Decl::Var(VarDecl {
+                    span: DUMMY_SP,
+                    kind: VarDeclKind::Var,
+                    declare: false,
+                    decls: self.vars.take(),
+                }))),
+            );
+        }
+
+        stmts
+    }
+
+    fn fold_stmts(&mut self, stmts: Vec<Stmt>) -> Vec<Stmt> {
+        let old_vars = self.vars.take();
+
+        let mut stmts = stmts.fold_children_with(self);
+        if !self.vars.is_empty() {
+            prepend(
+                &mut stmts,
+                Stmt::Decl(Decl::Var(VarDecl {
+                    span: DUMMY_SP,
+                    kind: VarDeclKind::Var,
+                    declare: false,
+                    decls: self.vars.take(),
+                })),
+            );
+        }
+
+        self.vars = old_vars;
+
+        stmts
+    }
+}
+
+impl InjectVars for Arrow {
+    fn take_vars(&mut self) -> Vec<VarDeclarator> {
+        self.vars.take()
+    }
 }
 
 struct ArgumentsReplacer {
@@ -226,6 +277,32 @@ impl Fold for ArgumentsReplacer {
     }
 }
 
+struct ThisReplacer<'a> {
+    to: &'a Ident,
+}
+
+impl VisitMut for ThisReplacer<'_> {
+    noop_visit_mut_type!();
+
+    fn visit_mut_arrow_expr(&mut self, _: &mut ArrowExpr) {}
+
+    fn visit_mut_constructor(&mut self, _: &mut Constructor) {}
+
+    fn visit_mut_expr(&mut self, e: &mut Expr) {
+        e.visit_mut_children_with(self);
+
+        match e {
+            Expr::This(..) => {
+                *e = Expr::Ident(self.to.clone());
+            }
+
+            _ => {}
+        }
+    }
+
+    fn visit_mut_function(&mut self, _: &mut Function) {}
+}
+
 #[derive(Default)]
 struct ArrowVisitor {
     found: bool,
