@@ -2,9 +2,9 @@ pub use self::{
     arrow::arrow, block_scoped_fn::block_scoped_functions, block_scoping::block_scoping,
     classes::classes, computed_props::computed_properties, destructuring::destructuring,
     duplicate_keys::duplicate_keys, for_of::for_of, function_name::function_name,
-    instanceof::instance_of, parameters::parameters, regenerator::regenerator,
-    shorthand_property::shorthand, spread::spread, sticky_regex::sticky_regex,
-    template_literal::template_literal, typeof_symbol::typeof_symbol,
+    instanceof::instance_of, new_target::new_target, parameters::parameters,
+    regenerator::regenerator, shorthand_property::shorthand, spread::spread,
+    sticky_regex::sticky_regex, template_literal::template_literal, typeof_symbol::typeof_symbol,
 };
 use serde::Deserialize;
 use swc_common::{chain, comments::Comments, Mark};
@@ -48,6 +48,7 @@ where
     chain!(
         block_scoped_functions(),
         template_literal(),
+        new_target(),
         classes(comments),
         spread(c.spread),
         function_name(),
