@@ -1,6 +1,6 @@
-use swc_common::DUMMY_SP;
+use swc_common::{util::take::Take, DUMMY_SP};
 use swc_ecma_ast::*;
-use swc_ecma_transforms_base::{ext::MapWithMut, perf::Check};
+use swc_ecma_transforms_base::perf::Check;
 use swc_ecma_transforms_macros::fast_path;
 use swc_ecma_utils::{alias_ident_for, prepend, ExprFactory};
 use swc_ecma_visit::{
