@@ -1,5 +1,5 @@
 use swc_ecma_transforms_compat::es2015::arrow;
-use swc_ecma_transforms_testing::test;
+use swc_ecma_transforms_testing::{compare_stdout, test};
 
 test!(
     ::swc_ecma_parser::Syntax::default(),
@@ -169,68 +169,51 @@ function test() {
 }"#,
     r#"
   function test() {
-    return (function(_arguments) {
+    var _arguments = arguments;
+    return function() {
       return _arguments[0];
-    }).bind(this, arguments)
+    }
   }"#
 );
 
-test!(
+compare_stdout!(
     ::swc_ecma_parser::Syntax::default(),
     |_| arrow(),
     arguments_nested_arrow,
-    r#"
-  function test() {
-    console.log(arguments[0]);
-    return () => {
+    "
+    function test() {
       console.log(arguments[0]);
       return () => {
-        console.log(arguments[0])
-      };
-    }
-  }"#,
-    r#"
-  function test() {
-    console.log(arguments[0]);
-    return (function(_arguments) {
-        console.log(_arguments[0]);
-        return function() {
-            console.log(_arguments[0]);
-        };
-    }).bind(this, arguments);
-  }"#
-);
-
-test!(
-    ::swc_ecma_parser::Syntax::default(),
-    |_| arrow(),
-    arguments_nested_fn,
-    r#"
-  function test() {
-    console.log(arguments[0]);
-    return () => {
-      console.log(arguments[0]);
-      return function() {
         console.log(arguments[0]);
         return () => {
           console.log(arguments[0])
         };
-      };
+      }
     }
-  }"#,
-    r#"
-  function test() {
-    console.log(arguments[0]);
-    return (function(_arguments) {
-        console.log(_arguments[0]);
+
+    test()(1)(2);
+    "
+);
+
+compare_stdout!(
+    ::swc_ecma_parser::Syntax::default(),
+    |_| arrow(),
+    arguments_nested_fn,
+    "
+    function test() {
+      console.log(arguments[0]);
+      return () => {
+        console.log(arguments[0]);
         return function() {
-            console.log(arguments[0]);
-            return (function(_arguments1) {
-                console.log(_arguments1[0]);
-            }).bind(this, arguments);
+          console.log(arguments[0]);
+          return () => {
+            console.log(arguments[0])
+          };
         };
-    }).bind(this, arguments);
-  }"#
+      }
+    }
+    test()(1)(2)(3);
+    "
 );
 
 test!(
@@ -268,3 +251,39 @@ test!(
     };
   }"#
 );
+
+test!(
+    ::swc_ecma_parser::Syntax::default(),
+    |_| arrow(),
+    issue_2212_1,
+    "const foo = () => this",
+    "
+    var _this = this;
+    const foo = function() {
+        return _this;
+    };
+    "
+);
+
+test!(
+    ::swc_ecma_parser::Syntax::default(),
+    |_| arrow(),
+    issue_2212_2,
+    "
+    const foo = function (){
+        () => () => () => this
+    }
+    ",
+    "
+    const foo = function() {
+      var _this = this;
+      (function() {
+          return function() {
+              return function() {
+                  return _this;
+              };
+          };
+      });
+    };
+    "
+);
